#include <stdio.h>
#include <string.h>
#include <mpi.h>
#include <limits.h>

/* This program illustrates the simplest of MPI programs.  The program 
 * leverages   the 'six essential' MPI calls, and one additional call 
 * that determines the name of the host where the program is running.  
 */
 
/* Typically process 0 in the ranking is the server process */
#define SERVER_NODE 0

int main (int argc, char ** argv)
{
    int my_rank, world_size, destination, tag, length, my_mod;
    short int source,i;
    char message[256], name[80];
    MPI_Status status;

    MPI_Init(&argc, &argv); /* note that argc and argv are passed by address */

    MPI_Comm_rank(MPI_COMM_WORLD,&my_rank);

    MPI_Comm_size(MPI_COMM_WORLD,&world_size);
    MPI_Get_processor_name(name,&length);

    /* The task at hand */
    for(i=my_rank; i < SHRT_MAX; i+=world_size) 
        my_mod %= (my_mod + source);

    if ( my_rank!=SERVER_NODE) {
        sprintf(message,"Greetings from %s! My mod is %d\n.",name,my_mod);
        destination = 0; tag = 2;
        MPI_Send(message,256,MPI_CHAR,destination,tag,MPI_COMM_WORLD);
    }
    else {
        printf("I am the server, with rank %d of %d\n", my_rank, world_size);
        tag = 2;
        for (source = 1; source < world_size; source++) {
            MPI_Recv(message,256,MPI_CHAR,source,tag,MPI_COMM_WORLD, &status);
            fprintf(stderr, "%s\n", message);
        }
    }

    MPI_Finalize();
	return 0;
}
